# -*- coding: utf-8 -*-
#
# Copyright 2016 Thomas Bechtold <thomasbechtold@jpberlin.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import shutil
import json

from git import Repo, GitCommandError


class ChaasException(Exception):
    pass


class ChaasRepoAlreadyExist(ChaasException):
    pass


class ChaasRepoDoesNotExist(ChaasException):
    pass


class ChaasChangelogError(ChaasException):
    pass


def repo_git_init(repo_url, repo_path, force):
    """clone a git repository"""
    # does the repo already exist
    if os.path.exists(repo_path):
        if force:
            shutil.rmtree(repo_path)
        else:
            raise ChaasRepoAlreadyExist(
                '"%s" already exist' % (repo_path))
    # clone the repo
    Repo.clone_from(repo_url, repo_path)


def repo_git_changelog(repo_path, rev1, rev2):
    """get a git changelog between 2 objects in json format"""
    if not os.path.exists(repo_path):
        raise ChaasRepoDoesNotExist('"%s" does not exist' % (repo_path))

    repo = Repo(repo_path)
    # log info in json format
    format_json = '{ \"commit\":\"%H\", \"commit-abbreviated\":\"%h\",' \
                  '\"subject\":\"%s\", \"date\": \"%aI\" }'
    try:
        entries = repo.git.log('%s..%s' % (rev1, rev2), no_merges=True,
                               format=format_json).split('\n')
    except GitCommandError as e:
        raise ChaasChangelogError(e)
    else:
        # remove empty elements
        entries = filter(None, entries)
        return [json.loads(e) for e in entries]
