#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright 2016 Thomas Bechtold <thomasbechtold@jpberlin.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from django.core.management.base import BaseCommand, CommandError
from django.db.models import Q

from django_chaas_backend import common
from django_chaas_backend.models import Repository


class Command(BaseCommand):
    help = 'Clone all not yet cloned repositories'

    def add_arguments(self, parser):
        parser.add_argument('repository', nargs='*', type=str,
                            help='Single or multiple repository ids. '
                            'If not given, handle all available repositories')
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Delete and clone if repository already exists',
        )

    def handle(self, *args, **options):
        # we ignore the current repo status when we have the --force switch
        if options['force']:
            repo_list = Repository.objects.all()
        else:
            repo_list = Repository.objects.filter(
                status=Repository.STATUS_UNKNOWN)

        # just handle some specific repos?
        if options['repository']:
            repo_list = repo_list.filter(
                Q(uuid__in=options['repository']) |
                Q(url__in=options['repository'])
            )

        for repo in repo_list:
            # set status
            repo.status = Repository.STATUS_CLONING
            repo.save()
            # clone repo
            self.stdout.write('Cloning "%s" into %s\n' % (repo.url, repo.path))
            try:
                common.repo_git_init(repo.url, repo.path, options['force'])
            except common.ChaasRepoAlreadyExist as e:
                repo.status = Repository.STATUS_CLONE_ERROR
                repo.save()
                raise CommandError(e)
            else:
                # set status
                repo.status = Repository.STATUS_CLONED
                repo.save()
            self.stdout.write('Cloned "%s" into %s\n' % (repo.url, repo.path))
